#version 330 core

layout (std140) uniform Material {
	vec4 diffuse;
	vec4 ambient;
	vec4 specular;
	vec4 emissive;
	float shininess;
	int texCount;
};

uniform sampler2DMS tex0;
uniform sampler2DMS tex1;

uniform float thickness;
uniform float threshold;
uniform int width;
uniform int height;

in vec2 fTexCoord;

out vec4 output;

void main() {
	mat3 gray;
	int i,j;

	vec2 offsetX = vec2(thickness, 0.0);
	vec2 offsetY = vec2(0.0, thickness);

	// we are filling our 3x3 gray matrix by averaging over the texture. 
	vec2 center = fTexCoord.st * vec2(width, height) - offsetY;
	for (int i = 0; i < 3; ++i) {
		//gray[i][0] = float(texture2D(tex0, center-offsetX).z); //dot(texture2D(texDepth, center - offsetX).rgb, vec3(0.33333));
		//gray[i][1] = float(texture2D(tex0, center).z); //dot(texture2D(texDepth, center).rgb, vec3(0.33333));
		//gray[i][2] = float(texture2D(tex0, center+offsetX).z); //dot(texture2D(texDepth, center + offsetX).rgb, vec3(0.33333));
		gray[i][0] = float(texelFetch(tex0, ivec2(center-offsetX),0).z); //dot(texture2D(texDepth, center - offsetX).rgb, vec3(0.33333));
		gray[i][1] = float(texelFetch(tex0, ivec2(center),1).z); //dot(texture2D(texDepth, center).rgb, vec3(0.33333));
		gray[i][2] = float(texelFetch(tex0, ivec2(center+offsetX),0).z); //dot(texture2D(texDepth, center + offsetX).rgb, vec3(0.33333));
		
		center += offsetY;
	}

	mat3 kernel;

	kernel[0][0] = -1.0; kernel[0][1] = -2.0; kernel[0][2] = -1.0;
	kernel[1][0] =  0.0; kernel[1][1] =  0.0; kernel[1][2] =  0.0;
	kernel[2][0] =  1.0; kernel[2][1] =  2.0; kernel[2][2] =  1.0;

	float sumX = 0.0;
	float sumY = 0.0;
	for (int i = 0; i < 3; ++i) {
		for (int j = 0; j < 3; ++j) {
			sumX += kernel[i][j] * gray[i][j];
			sumY += kernel[j][i] * gray[i][j];
		}
	}
	float result = sqrt(sumX * sumX + sumY * sumY);

	if (result > threshold) {
		output = vec4(0.0, 0.0, 0.0, 1.0);
	} else if (result > 0.5 * threshold) {
		float diff = (result - (threshold * 0.5)) / (threshold - (threshold * 0.5));
		for (int i = 0; i < 4; ++i) {
			output += texelFetch(tex1, ivec2(fTexCoord.st * vec2(width, height)), i);
		}
		output = output / 4;
		output = mix(output, vec4(0.0, 0.0, 0.0, 1.0), diff);
	} else {
		output = vec4(0);
		for (int i = 0; i < 4; ++i) {
			output += texelFetch(tex1, ivec2(fTexCoord.st * vec2(width, height)), i);
		}
		output = output / 4;
	}
	//output = vec4(result, result, result, 1.0);
}