#version 330

uniform sampler2D texUnit;
uniform sampler2DMS depthUnit;
uniform sampler2DMS colorUnit;

uniform bool fadeEffect;
uniform int width;
uniform int height;

in float fLifetime;
in vec2 fTexCoord;
in vec4 fPosition;		// gl_FragCoord is just strange... just use a new variable here

out vec4 output;

void main()
{
	// 'constants' for the near clipping plane and far clipping plane
	float n = 0.1;
	float f = 100.0;

	if (fLifetime < 0.0) {	// is the particle already dead?!
		discard;
	}

	// fPosition is in NDC if we homogenize the vector: i.e. [-1, +1] for xyz
	// so we need to transform coordinates for texture lookup here
	vec2 texCoord = ((fPosition.xy / fPosition.w) * 0.5 + vec2(0.5));
	float depth = texelFetch(depthUnit, ivec2(texCoord * vec2(width, height)), 4).r;	// lookup the depth value
	float z_n = 2.0 * depth - 1.0;					// get the normalized device coordinates
	depth = 2.0 * (f * n) / (f + n - z_n * (f - n));	// get the depth value between [n, f]

	// we gonna throw away the pixel if the position is farther away from the camera than
	// an other object in the depth buffer
	if (depth < fPosition.w) {
		discard;
	}

	// regular particle shading...
	output = texture2D(texUnit, fTexCoord);
	if (fadeEffect == true) {
		output.a *= fLifetime;
	}
}