#version 330

layout(points) in;
layout(triangle_strip, max_vertices=4) out;

uniform mat4 M;
uniform mat4 V;
uniform mat4 P;
uniform vec3 cameraLocation;

uniform float particleSize;
in float gLifetime[1];
in vec2 gTexCoord[1];

uniform bool scaleEffect;
uniform bool randomRotateEffect;

out float fLifetime;
out vec2 fTexCoord;
out vec4 fPosition;

float rand(vec2 co){
    return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
}

mat4 billboardMatrixForVector(vec3 pos)
{
	vec3 look = cameraLocation - pos.xyz;
	look = normalize(look);
	vec3 up = vec3(0.0, 1.0, 0.0); // assume camera is always looking up!
	vec3 right = normalize(cross(up, look));
	up = cross(look, right);
	up = up * 0.2;
	right = right * 0.2;
	look = look * 0.2;
		
	return transpose(mat4(right.x, up.x, look.x, pos.x, right.y, up.y, look.y, pos.y, right.z, up.z, look.z, pos.z, 0, 0, 0, 1));
}

void main()
{	
	fLifetime = gLifetime[0];

	vec4 objectPos = gl_in[0].gl_Position;
	vec4 worldPos = M * gl_in[0].gl_Position;

	float pSize = particleSize;

	if (scaleEffect == true) {
		pSize = max(0.0, exp(gLifetime[0] * particleSize) - 1.0);
	}

	mat4 objToWorld = mat4(1.0, 0.0, 0.0, 0.0, 
						   0.0, 1.0, 0.0, 0.0,
						   0.0, 0.0, 1.0, 0.0,
						   0.0, 0.0, 0.0, 1.0);
	
	if (randomRotateEffect == true) {
		float angle = gLifetime[0] * 10 + objectPos.x * objectPos.z * 10;
		objToWorld = 
		mat4( cos( angle ), -sin( angle ), 0.0, 0.0,
			  sin( angle ),  cos( angle ), 0.0, 0.0,
			  0.0,           0.0, 1.0, 0.0,
			  0.0,           0.0, 0.0, 1.0 );
	}

	mat4 matInvObjPos = mat4(
		1.0, 0.0, 0.0, 0.0, 
		0.0, 1.0, 0.0, 0.0,
		0.0, 0.0, 1.0, 0.0,
		-objectPos.x, -objectPos.y, -objectPos.z, 1.0);
	mat4 matObjPos = mat4(
		1.0, 0.0, 0.0, 0.0, 
		0.0, 1.0, 0.0, 0.0,
		0.0, 0.0, 1.0, 0.0,
		objectPos.x, objectPos.y, objectPos.z, 1.0);
	mat4 Mnew = billboardMatrixForVector(worldPos.xyz / worldPos.w) * matObjPos * objToWorld * matInvObjPos;
	//gl_Position = P * V * Mnew * (objectPos - vec4(-particleSize, -particleSize, 0.0, 0.0));
	fPosition = P * V * Mnew * (objectPos - vec4(-pSize, -pSize, 0.0, 0.0));
	gl_Position = fPosition;
	fTexCoord = vec2(1.0, 1.0);
	EmitVertex();
	
	//gl_Position = P * V * Mnew * (objectPos - vec4(particleSize, -particleSize, 0.0, 0.0));
	fPosition = P * V * Mnew * (objectPos - vec4(pSize, -pSize, 0.0, 0.0));
	gl_Position = fPosition;
	fTexCoord = vec2(0.0, 1.0);
	EmitVertex();

	//gl_Position = P * V * Mnew * (objectPos - vec4(-particleSize, particleSize, 0.0, 0.0));
	fPosition = P * V * Mnew * (objectPos - vec4(-pSize, pSize, 0.0, 0.0));
	gl_Position = fPosition;
	fTexCoord = vec2(1.0, 0.0);
	EmitVertex();
	
	//gl_Position = P * V * Mnew * (objectPos - vec4(particleSize, particleSize, 0.0, 0.0));
	fPosition = P * V * Mnew * (objectPos - vec4(pSize, pSize, 0.0, 0.0));
	gl_Position = fPosition;
	fTexCoord = vec2(0.0, 0.0);
	EmitVertex();

	EndPrimitive();
}
