#version 330 core
layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 texCoord;
layout(location = 3) in vec2 boneWeights;
layout(location = 4) in ivec2 boneIndices;

uniform mat4 M;
uniform mat4 Minv;
uniform mat4 V;
uniform mat4 Vinv;
uniform mat4 P;
uniform mat4 boneMatrices[10];

uniform int numBones;
uniform int isBillboard;
uniform int applyHeightMap;

uniform vec2 worldMin;
uniform vec2 worldMax;
uniform float minHeight;
uniform float maxHeight;
uniform float heightmapScaleWidth;
uniform float heightmapScaleHeight;

uniform vec3 cameraLocation;

uniform sampler2D heightUnit;

out vec3 fPosition;
out vec2 fTexCoord;

vec4 translateHeight(vec4 pos) 
{
	vec2 clampedPos = clamp(pos.xz, worldMin, worldMax);
	vec2 texCoord = clampedPos / (worldMax - worldMin) * vec2(heightmapScaleWidth, heightmapScaleHeight);
	texCoord.y = texCoord.y;
	float h = texture2D(heightUnit,  texCoord.st).a;
	pos.y = h * (maxHeight - minHeight) + minHeight;
	return pos;
}

void main()
{
	mat4 MV = V * M;

	vec4 p = vec4(position, 1.0);
	vec4 n = vec4(normal, 0.0);

	if (boneIndices.x != -1 && numBones > 0) {
		p = boneMatrices[boneIndices.x] * boneWeights[boneIndices.x] * vec4(position, 1.0);
		n = boneMatrices[boneIndices.x] * boneWeights[boneIndices.x] * vec4(normal, 0.0);
	}
	if (boneIndices.y != -1 && numBones > 0) {
		p += boneMatrices[boneIndices.y] * boneWeights[boneIndices.y] * vec4(position, 1.0);
		n += boneMatrices[boneIndices.y] * boneWeights[boneIndices.y] * vec4(normal, 0.0);
	}

	if (isBillboard == 1) {
		vec3 position = vec3(M[3][0], M[3][1], M[3][2]);
		vec3 look = cameraLocation - position;
		//look.x = 0.0;
		look = normalize(look);
		vec3 up = vec3(0.0, 1.0, 0.0); // assume camera is always looking up!
		vec3 right = normalize(cross(up, look));
		up = cross(look, right);
		up = up * 0.2;
		right = right * 0.2;
		look = look * 0.2;
		
		mat4 Mnew = transpose(mat4(right.x, up.x, look.x, M[3][0], right.y, up.y, look.y, M[3][1], right.z, up.z, look.z, M[3][2], 0, 0, 0, 1));
		vec4 worldP = Mnew * p;

		if (applyHeightMap == 1) {
			worldP = translateHeight(worldP);
		}

		gl_Position = P * V * worldP;
		n = transpose(inverse(Mnew) * Vinv) * n; // TODO: make this correct for heightmaps
		vec4 tmp = V * worldP;
		fPosition = tmp.xyz / tmp.w;
	} else {
		vec4 worldP = M * p;
		
		if (applyHeightMap == 1) {
			worldP = translateHeight(worldP);
		}

		vec4 tmp = V * worldP;
		fPosition = tmp.xyz / tmp.w;
		gl_Position = P * V * worldP;
		n = transpose(Minv * Vinv) * n;	// TODO: make this correct for heightmaps
	}
	
	fTexCoord = vec2(texCoord);
}